#include <DateUtils.hpp>
#include <math.h>
#include <vector.h>

#define VERSION "1.20"

//#define DEBUG

//-------------------------------------------------------------------------
//M A T H E M A T I K
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//Winkelkonstanten und -funktionen
//-------------------------------------------------------------------------

const float pi = 3.1415927f;
const float pi2 = 2 * pi;
const float pi_2 = pi/2.0f;
const float pi_180 = pi / 180.0f;
const float w180_pi = 180.0f / pi;
const float vollwinkel = 360.0f;

float degInRad(float winkel)
{
  return winkel * pi_180;
}

float radInDeg(float winkel)
{
  return winkel * w180_pi;
}

//-------------------------------------------------------------------------
//Vektorklassen und -funktionen
//-------------------------------------------------------------------------
class Vektor2D
{
  public:
    float x,y;
  public:
    Vektor2D();
    Vektor2D(float neuX, float neuY);
};

Vektor2D::Vektor2D()
{
  x = 0;
  y = 0;
}

Vektor2D::Vektor2D(float neuX, float neuY)
{
  x = neuX;
  y = neuY;
}

Vektor2D operator+(Vektor2D v1, Vektor2D v2)
{
  return Vektor2D(v1.x + v2.x, v1.y + v2.y);
}

Vektor2D operator-(Vektor2D v1, Vektor2D v2)
{
  return Vektor2D(v1.x - v2.x, v1.y - v2.y);
}

Vektor2D operator-(Vektor2D v)
{
  return Vektor2D(-v.x, -v.y);
}

Vektor2D operator*(float f, Vektor2D v)
{
  return Vektor2D(f * v.x, f * v.y);
}

Vektor2D operator*(Vektor2D v, float f)
{
  return Vektor2D(f * v.x, f * v.y);
}

Vektor2D operator/(Vektor2D v, float f)
{
  return Vektor2D(v.x / f, v.y / f);
}

float operator*(Vektor2D v1, Vektor2D v2)
{
  return v1.x * v2.x + v1.y * v2.y;
}

bool operator==(Vektor2D v1, Vektor2D v2)
{
  return (v1.x == v2.x) && (v1.y == v2.y);
}

bool operator!=(Vektor2D v1, Vektor2D v2)
{
  return !(v1 == v2);
}

float betrag(Vektor2D v)
{
  return sqrt(v * v);
}

float betragHoch2(Vektor2D v)
{
  return v*v;
}

float winkel(Vektor2D v)
{
  if (v.x == 0)
  {
    if (v.y > 0)
    {
      return pi_2;
    }
    else
    {
      return -pi_2;
    }
  }

  float resultat = atan(v.y / v.x);
  if (v.x < 0)
  {
    resultat = resultat + pi;
  }
  return resultat;
}

Vektor2D einheitsvektor(Vektor2D v)
{
  float b = betrag(v);
  if (b > 0)
  {
	return v/b;
  }
  else
  {
	return v;
  }
}

Vektor2D einheitsvektor(float winkel)
{
  return Vektor2D(cos(winkel), sin(winkel));
}

Vektor2D senkrecht(Vektor2D v)
{
  return Vektor2D(-v.y, v.x);
}

//-------------------------------------------------------------------------
//Lineare Gleichungen
//-------------------------------------------------------------------------
const float fastnull = 1.0e-6;

bool istNull(float wert)
{
  return fabs(wert) < fastnull;
}

bool istNull(Vektor2D v)
{
  return istNull(v.x) && istNull(v.y);
}

bool linearAbhaengig(Vektor2D v1, Vektor2D v2)
{
  return istNull(v1.x * v2.y - v1.y * v2.x);
}

class Loesung  //Lsung k1,k2 von Gleichungen der Art v = k1*v1 + k2*v2
{
  public:
	float k1;
	float k2;
	bool loesbar;   //lsbar?
	bool eindeutig; //genau eine Lsung?
  public:
	Loesung(float neuK1, float neuK2, bool neuLoesbar, bool neuEindeutig);
};

Loesung::Loesung(float neuK1, float neuK2, bool neuLoesbar,
				 bool neuEindeutig)
{
  k1 = neuK1;
  k2 = neuK2;
  loesbar = neuLoesbar;
  eindeutig = neuEindeutig;
}

Loesung loese(Vektor2D v, Vektor2D v1, Vektor2D v2)
{
  //gibt die Lsungen k1 der Gleichung: v = k1*v1 + k2*v2
  float det = v1.x * v2.y - v1.y * v2.x;
  if (istNull(det))
  {
	if(istNull(v1))
	{
	  if(istNull(v2))
	  {
		return Loesung(NaN, NaN, istNull(v), false);
	  }
	  else
	  {
		if(linearAbhaengig(v, v2))
		{
		  if(istNull(v2.x))
		  {
			return Loesung(NaN, v.y / v2.y, true, false);
		  }
		  else
		  {
			return Loesung(NaN, v.x / v2.x, true, false);
		  }
		}
		else
		{
		  return Loesung(NaN, NaN, false, false);
		}
	  }
	}
	else
	{
	  if(linearAbhaengig(v, v1))
	  {
		if(istNull(v1.x))
		{
		  return Loesung(v.y / v1.y, NaN, true, false);
		}
		else
		{
		  return Loesung(v.x / v1.x, NaN, true, false);
		}
	  }
	  else
	  {
		return Loesung(NaN, NaN, false, false);
	  }
	}
  }
  else
  {
	float det1 = v.x * v2.y - v.y * v2.x;
	float det2 = v.y * v1.x - v.x * v1.y;

	float kehrwertDet = 1.0f / det;

	return Loesung(det1 * kehrwertDet, det2 * kehrwertDet, true, true);
  }
}

//-------------------------------------------------------------------------
//G R A F I K
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//Zeichenflaeche 2D
//-------------------------------------------------------------------------
class Zeichenflaeche2D
{
  public:
	TImage *image;
	TCanvas *canvas;
	float xmin, xmax;
	float ymin, ymax;
	float deltaX, deltaY;

  public:
	Zeichenflaeche2D(TImage *neuImage, float neuXMin, float neuXMax,
					 float neuYMin, float neuYMax);
	Zeichenflaeche2D(TImage *neuImage, float neuXMin, float neuXMax,
					 float neuYMin);
	void aktiviereStift(bool neuStatus);
	void setzeStiftModus(TPenMode modus);
	void setzeStiftFarbe(TColor farbe);
	void setzeStiftDicke(int dicke);
	void aktiviereFuellen(bool neuStatus);
	void setzeFuellenFarbe(TColor farbe);

	void loescheZFlaeche();

	int zFlaecheXInImageX(float zx);
	int zFlaecheYInImageY(float zy);
	float imageXInZFlaecheX(int ix);
	float imageYInZFlaecheY(int iy);

	int zFlaecheDXInImageDX(float zdx);
	int zFlaecheDYInImageDY(float zdy);
	float imageDXInZFlaecheDX(int idx);
	float imageDYInZFlaecheDY(int idy);

	void zeichneAchsen(float deltaX, float deltaY);
	void zeichneXAchse(float deltaX);
	void zeichneYAchse(float deltaY);

	void zeichnePunkt(Vektor2D v);
	void zeichneLinie(Vektor2D v1, Vektor2D v2);
	void zeichneLinienzug(Vektor2D v[], int zahlPunkte);
	void zeichneVieleck(Vektor2D v[], int zahlPunkte);
	void zeichneRechteck(Vektor2D v1, Vektor2D v2);
	void zeichneKreis(Vektor2D v, float r);
	void zeichneEllipse(Vektor2D v, float rx, float ry);

	void zeichneGrafik(TGraphic *grafik, Vektor2D v);
	void zeichneGrafik(TGraphic *grafik, Vektor2D v, float groesse);
	void zeichneGrafik(TGraphic *grafik, Vektor2D v,
					   float groesseX, float groesseY);
	void zeichneBildAusListe(TImageList *liste, int index, Vektor2D v);
	void zeichneBildAusListe(TImageList *liste, int index, Vektor2D v,
							 float groesse);
	void zeichneBildAusListe(TImageList *liste, int index, Vektor2D v,
							 float groesseX, float groesseY);

	void zeichneFeder(Vektor2D v1, Vektor2D v2, int zahlSegmente);
	void zeichneSeil(Vektor2D v1, Vektor2D v2, float laenge);

	void positioniereBild(TImage *posImage, Vektor2D v);
	void positioniereBild(TImage *posImage, Vektor2D v, float groesse);
	void positioniereBild(TImage *posImage, Vektor2D v, float groesseX,
						  float groesseY);
};

Zeichenflaeche2D::Zeichenflaeche2D(TImage *neuImage, float neuXMin,
					 float neuXMax, float neuYMin, float neuYMax)
{
  image =neuImage;
  canvas = neuImage->Canvas;

  xmin = neuXMin;
  xmax = neuXMax;
  ymin = neuYMin;
  ymax = neuYMax;

  deltaX = imageDXInZFlaecheDX(1);
  deltaY = imageDYInZFlaecheDY(1);

  loescheZFlaeche();
}

Zeichenflaeche2D::Zeichenflaeche2D(TImage *neuImage,
					 float neuXMin, float neuXMax, float neuYMin)
{
  image =neuImage;
  canvas = neuImage->Canvas;

  xmin = neuXMin;
  xmax = neuXMax;
  ymin = neuYMin;
  ymax = ymin + (neuXMax - neuXMin) * image->Height / image->Width;

  deltaX = imageDXInZFlaecheDX(1);
  deltaY = deltaX;

  loescheZFlaeche();
}

void Zeichenflaeche2D::aktiviereStift(bool status)
{
  if (status)
  {
	canvas->Pen->Style = psSolid;
	setzeStiftModus(pmCopy);
  }
  else
  {
	canvas->Pen->Style = psClear;
  }
}

void Zeichenflaeche2D::setzeStiftModus(TPenMode modus)
{
  canvas->Pen->Mode = modus;
}

void Zeichenflaeche2D::setzeStiftFarbe(TColor farbe)
{
  canvas->Pen->Color = farbe;
}

void Zeichenflaeche2D::setzeStiftDicke(int dicke)
{
  canvas->Pen->Width = dicke;
}

void Zeichenflaeche2D::aktiviereFuellen(bool status)
{
  if (status)
  {
	canvas->Brush->Style = bsSolid;
  }
  else
  {
	canvas->Brush->Style = bsClear;
  }
}

void Zeichenflaeche2D::setzeFuellenFarbe(TColor farbe)
{
  canvas->Brush->Color = farbe;
}

void Zeichenflaeche2D::loescheZFlaeche()
{
  TPenStyle altPenStyle = canvas->Pen->Style;
  TBrushStyle altBrushStyle = canvas->Brush->Style;
  TColor altBrushFarbe = canvas->Brush->Color;

  aktiviereStift(false);
  aktiviereFuellen(true);
  setzeStiftModus(pmCopy);
  setzeFuellenFarbe(clWhite);
  canvas->Rectangle(0, 0, image->Width, image->Height);

  canvas->Pen->Style = altPenStyle;
  canvas->Brush->Style = altBrushStyle;
  setzeFuellenFarbe(altBrushFarbe);
}

int Zeichenflaeche2D::zFlaecheXInImageX(float zx)
{
  return (zx-xmin) / (xmax-xmin) * image->Width;
}

int Zeichenflaeche2D::zFlaecheYInImageY(float zy)
{
  return (ymax-zy) / (ymax-ymin) * image->Height;
}

float Zeichenflaeche2D::imageXInZFlaecheX(int ix)
{
  return xmin + (xmax - xmin) * ix / image->Width;
}

float Zeichenflaeche2D::imageYInZFlaecheY(int iy)
{
  return ymax - (ymax - ymin) * iy / image->Height;
}

int Zeichenflaeche2D::zFlaecheDXInImageDX(float zdx)
{
  return zdx / (xmax-xmin) * image->Width;
}

int Zeichenflaeche2D::zFlaecheDYInImageDY(float zdy)
{
  return zdy / (ymax-ymin) * image->Height;
}

float Zeichenflaeche2D::imageDXInZFlaecheDX(int idx)
{
  return (xmax - xmin) * idx / image->Width;
}

float Zeichenflaeche2D::imageDYInZFlaecheDY(int idy)
{
  return (ymax - ymin) * idy / image->Height;
}

void Zeichenflaeche2D::zeichneXAchse(float deltaX)
{
  setzeStiftModus(pmCopy);
  setzeStiftFarbe(clGray);
  setzeStiftDicke(1);
  int iy = zFlaecheYInImageY(0.0f);
  if (iy >= 0 && iy < image->Height)
  {
	float x;
	int ix;

	canvas->MoveTo(0, iy);
	canvas->LineTo(image->Width, iy);

	x=0.0f;
	while(x < xmax)
	{
	  if(x > xmin)
	  {
		ix = zFlaecheXInImageX(x);
		canvas->MoveTo(ix, iy-4);
		canvas->LineTo(ix, iy+4);
	  }
	  x = x + deltaX;
	}

	x = -deltaX;
	while(x > xmin)
	{
	  if(x < xmax)
	  {
		ix = zFlaecheXInImageX(x);
		canvas->MoveTo(ix, iy-4);
		canvas->LineTo(ix, iy+4);
	  }
	  x = x - deltaX;
	}

  }
}

void Zeichenflaeche2D::zeichneYAchse(float deltaY)
{
  setzeStiftModus(pmCopy);
  setzeStiftFarbe(clGray);
  setzeStiftDicke(1);
  int ix = zFlaecheXInImageX(0.0f);
  if (ix >= 0 && ix < image->Width)
  {
	float y;
	int iy;
	canvas->MoveTo(ix, 0);
	canvas->LineTo(ix, image->Height);

	y=0.0f;
	while(y < ymax)
	{
	  if(y > ymin)
	  {
		iy = zFlaecheYInImageY(y);
		canvas->MoveTo(ix-4, iy);
		canvas->LineTo(ix+4, iy);
	  }
	  y = y + deltaY;
	}

	y = -deltaY;
	while(y > ymin)
	{
	  if(y < ymax)
	  {
		iy = zFlaecheYInImageY(y);
		canvas->MoveTo(ix-4, iy);
		canvas->LineTo(ix+4, iy);
	  }
	  y = y - deltaY;
	}

  }
}

void Zeichenflaeche2D::zeichneAchsen(float deltaX, float deltaY)
{
  zeichneXAchse(deltaX);
  zeichneYAchse(deltaY);
}

void Zeichenflaeche2D::zeichnePunkt(Vektor2D v)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);

  canvas->Pixels[ix][iy] = clBlack;
}

void Zeichenflaeche2D::zeichneLinie(Vektor2D v1, Vektor2D v2)
{
  int ix1 = zFlaecheXInImageX(v1.x);
  int iy1 = zFlaecheYInImageY(v1.y);
  int ix2 = zFlaecheXInImageX(v2.x);
  int iy2 = zFlaecheYInImageY(v2.y);

  canvas->MoveTo(ix1, iy1);
  canvas->LineTo(ix2, iy2);
}

void Zeichenflaeche2D::zeichneLinienzug(Vektor2D v[], int zahlPunkte)
{
  if (zahlPunkte > 1)
  {
	int i;
	Vektor2D v0 = v[0];
	for (i=1; i<zahlPunkte; i++)
	{
	  zeichneLinie(v0, v[i]);
	  v0 = v[i];
	}
  }
}

void Zeichenflaeche2D::zeichneVieleck(Vektor2D v[], int zahlPunkte)
{
  if (zahlPunkte > 1)
  {
	int i, ix, iy;
	TPoint *listePunkte;

	listePunkte = new TPoint[zahlPunkte];

	for (i=0; i<zahlPunkte; i++)
	{
	  ix = zFlaecheXInImageX(v[i].x);
	  iy = zFlaecheYInImageY(v[i].y);
	  listePunkte[i] = TPoint(ix, iy);
	}

	canvas->Polygon(listePunkte, zahlPunkte-1);

	delete [] listePunkte;
  }
}

void Zeichenflaeche2D::zeichneKreis(Vektor2D v, float r)
{
  zeichneEllipse(v, r, r);
}

void Zeichenflaeche2D::zeichneEllipse(Vektor2D v, float rx, float ry)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int irx = zFlaecheDXInImageDX(rx);
  int iry = zFlaecheDYInImageDY(ry);

  canvas->Ellipse(ix - irx, iy - iry, ix + irx, iy + iry);
}

void Zeichenflaeche2D::zeichneRechteck(Vektor2D v1, Vektor2D v2)
{
  int ix1 = zFlaecheXInImageX(v1.x);
  int iy1 = zFlaecheYInImageY(v1.y);
  int ix2 = zFlaecheXInImageX(v2.x);
  int iy2 = zFlaecheYInImageY(v2.y);

  canvas->Rectangle(ix1, iy1, ix2, iy2);
}

void Zeichenflaeche2D::zeichneFeder(Vektor2D v1, Vektor2D v2,
								   int zahlSegmente)
{
  Vektor2D abschnitt = (v2 - v1) / (zahlSegmente * 3);
  Vektor2D sAbschnitt = 0.25f * einheitsvektor(senkrecht(abschnitt));
  Vektor2D punkt1 = v1;
  Vektor2D zick, zack, punkt2;

  for (int i=0; i<zahlSegmente; i++)
  {
	zick = punkt1 + abschnitt + sAbschnitt;
	zack = punkt1 + 2 * abschnitt - sAbschnitt;
	punkt2 = punkt1 + 3 * abschnitt;

	zeichneLinie(punkt1, zick);
	zeichneLinie(zick, zack);
	zeichneLinie(zack, punkt2);

	punkt1 = punkt2;
  }
}

void Zeichenflaeche2D::zeichneSeil(Vektor2D v1, Vektor2D v2, float laenge)
{
  float entfernung = betrag(v2 - v1);
  if (entfernung >= 0.999 * laenge)
  {
	zeichneLinie(v1, v2);
  }
  else
  {
	const float teile = 8;
	const float teileMitte = teile/2;
	float teileAbstandVonMitte;
	float durchbiegung = 0.5f * (laenge - entfernung);

	Vektor2D punkt1 = v1;
	Vektor2D punkt2;
	float x0 = v1.x;
	float deltaX = v2.x - x0;
	float y0 = v1.y;
	float deltaY = v2.y - y0;

	for(int i=1; i<=teile; i++)
	{
	  teileAbstandVonMitte = i - teileMitte;
	  punkt2 = Vektor2D(x0 + deltaX * i / teile,
						y0 + deltaY * i / teile - durchbiegung *
						(1.0f -
						 (teileAbstandVonMitte * teileAbstandVonMitte)
						 / (teileMitte * teileMitte))
					   );
	  zeichneLinie(punkt1, punkt2);
	  punkt1 = punkt2;
	}
  }
}

void Zeichenflaeche2D::zeichneGrafik(TGraphic *grafik, Vektor2D v)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = grafik->Width;
  int idy = grafik->Height;

  image->Canvas->Draw(ix - idx / 2, iy - idy / 2, grafik);
}

void Zeichenflaeche2D::zeichneGrafik(TGraphic *grafik, Vektor2D v,
									 float groesse)
{
  zeichneGrafik(grafik, v, groesse,
				groesse * grafik->Height / grafik->Width);
}

void Zeichenflaeche2D::zeichneGrafik(TGraphic *grafik, Vektor2D v,
									 float groesseX, float groesseY)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = zFlaecheDXInImageDX(groesseX);
  int idy = zFlaecheDYInImageDY(groesseY);

  image->Canvas->StretchDraw(Rect(ix - idx / 2, iy - idy / 2,
								  ix + idx / 2, iy + idy / 2), grafik);
}

void Zeichenflaeche2D::zeichneBildAusListe(TImageList *liste, int index,
										   Vektor2D v)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = liste->Width;
  int idy = liste->Height;

  liste->Draw(canvas, ix - idx / 2, iy - idy / 2, index, true);
  image->Refresh();
}

void Zeichenflaeche2D::zeichneBildAusListe(TImageList *liste, int index,
										   Vektor2D v, float groesse)
{
  zeichneBildAusListe(liste, index, v, groesse,
					  groesse * liste->Height / liste->Width);
}

void Zeichenflaeche2D::zeichneBildAusListe(TImageList *liste, int index,
										   Vektor2D v, float groesseX,
										   float groesseY)
{
  Graphics::TBitmap *bitmap = new Graphics::TBitmap();

  liste->GetBitmap(index, bitmap);

  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = zFlaecheDXInImageDX(groesseX);
  int idy = zFlaecheDYInImageDY(groesseY);

  image->Canvas->StretchDraw(Rect(ix - idx / 2, iy - idy / 2,
								  ix + idx / 2, iy + idy / 2), bitmap);
}

void Zeichenflaeche2D::positioniereBild(TImage *posImage, Vektor2D v)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = posImage->Width;
  int idy = posImage->Height;

  if (ix - idx/2 >= 0 && ix + idx/2 <= image->Width &&
	  iy - idy/2 >= 0 && iy + idy/2 <= image->Height)
  {
	posImage->Left = image->Left + ix - idx / 2;
	posImage->Top = image->Top + iy - idy / 2;
	posImage->Visible = true;
  }
  else
  {
	posImage->Visible = false;
  }
}

void Zeichenflaeche2D::positioniereBild(TImage *posImage, Vektor2D v,
										float groesse)
{
  positioniereBild(posImage, v, groesse,
				   groesse / posImage->Width * posImage->Height);
}

void Zeichenflaeche2D::positioniereBild(TImage *posImage, Vektor2D v,
										float groesseX, float groesseY)
{
  int ix = zFlaecheXInImageX(v.x);
  int iy = zFlaecheYInImageY(v.y);
  int idx = zFlaecheDXInImageDX(groesseX);
  int idy = zFlaecheDYInImageDY(groesseY);

  if (ix - idx/2 >= 0 && ix + idx/2 <= image->Width &&
	  iy - idy/2 >= 0 && iy + idy/2 <= image->Height)
  {
	posImage->Stretch = false;
	posImage->Width = idx;
	posImage->Height = idy;
	posImage->Left = image->Left + ix - idx / 2;
	posImage->Top = image->Top + iy - idy / 2;
	posImage->Stretch = true;
	posImage->Visible = true;
  }
  else
  {
	posImage->Visible = false;
  }
}

//-------------------------------------------------------------------------
//P H Y S I K
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
//Physikklassen fr Umrissformen
//-------------------------------------------------------------------------
const int halbebene = 0;
const int kreis = 1;

class Umrissform
{
  public:
	int typ;
	float abprallfaktor;
  public:
	virtual bool liegtInnerhalb(Vektor2D relOrt) = 0;
};

class Halbebene : public Umrissform
{
  public:
	Vektor2D pkt1;
	Vektor2D pkt2;
	float dicke;

	float laenge;
	Vektor2D pkt1ZuPkt2;
	Vektor2D pkt1ZuPkt2Einheit;
	Vektor2D normale;
  public:
	Halbebene(Vektor2D neuPkt1, Vektor2D neuPkt2, float neuDicke);
	bool liegtInnerhalb(Vektor2D relOrt);
};

Halbebene::Halbebene(Vektor2D neuPkt1, Vektor2D neuPkt2, float neuDicke)
{
  pkt1 = neuPkt1;
  pkt2 = neuPkt2;
  dicke = neuDicke;
  typ = halbebene;

  pkt1ZuPkt2 = pkt2 - pkt1;
  laenge = betrag(pkt1ZuPkt2);
  pkt1ZuPkt2Einheit = pkt1ZuPkt2 / laenge;
  normale = senkrecht(pkt1ZuPkt2Einheit);
}

bool Halbebene::liegtInnerhalb(Vektor2D relOrt)
{
  Vektor2D pkt1ZuRelOrt = relOrt - pkt1;
  float projAbstand = pkt1ZuRelOrt * pkt1ZuPkt2Einheit;

  bool resultat1 = projAbstand > 0.0f || projAbstand < -dicke;;

  if (!resultat1)
  {
	return false;
  }

  float skalarprodukt = pkt1ZuRelOrt * pkt1ZuPkt2;
  float betragQuadrat = betragHoch2(pkt1ZuPkt2);

  bool resultat2 = (skalarprodukt >= 0.0f) &&
				   (skalarprodukt <= betragQuadrat);
  return resultat2;
}

class Kreis : public Umrissform
{
  public:
	float radius;
  public:
	Kreis(float neuRadius);
	bool liegtInnerhalb(Vektor2D relOrt);
};

Kreis::Kreis(float neuRadius)
{
  radius = neuRadius;
  typ = kreis;
}

bool Kreis::liegtInnerhalb(Vektor2D relOrt)
{
  return betragHoch2(relOrt) < radius * radius;
}

//-------------------------------------------------------------------------
//Physikklassen fr Objekte
//-------------------------------------------------------------------------
const Vektor2D gravBeschl(0, -9.81);
const Vektor2D nullvektor(0,0);

class Objekt
{
  public:
	Zeichenflaeche2D* zFlaeche;
	TImage* image;

	Vektor2D ort;
	Vektor2D geschw;
	Vektor2D beschl;

	float masse;
	float rueckprallFaktor;
	Umrissform* umrissform;

  public:
	virtual void bewegeDich(float zeitdauer) = 0;
	virtual void beeinflusstVonKraft(Vektor2D kraft) = 0;

	virtual void setzeOrt(Vektor2D neuOrt) = 0;
	virtual void setzeGeschw(Vektor2D neuGeschw) = 0;
	virtual void setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw) = 0;

	virtual void loescheDichWeg() = 0;
	virtual void zeigeDichAn() = 0;
};

class Teilchen : public Objekt
{
  public:
	Teilchen(Zeichenflaeche2D *neuZFlaeche, TImage *neuImage,
			 float neuMasse, float neuRadius, float neuRueckprallFaktor,
			 Vektor2D neuOrt,
			 Vektor2D neuGeschw);
	~Teilchen();

	void bewegeDich(float zeitdauer);
	void beeinflusstVonKraft(Vektor2D kraft);

	void setzeOrt(Vektor2D neuOrt);
	void setzeGeschw(Vektor2D neuGeschw);
	void setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw);

	void loescheDichWeg();
	void zeigeDichAn();
};

Teilchen::Teilchen(Zeichenflaeche2D *neuZFlaeche, TImage *neuImage,
				   float neuMasse, float neuRadius,
				   float neuRueckprallFaktor,
				   Vektor2D neuOrt,
				   Vektor2D neuGeschw)
{
  zFlaeche = neuZFlaeche;
  image = neuImage;

  masse = neuMasse;
  rueckprallFaktor = neuRueckprallFaktor;
  umrissform = new Kreis(neuRadius);

  ort = neuOrt;
  geschw = neuGeschw;
  beschl = nullvektor;

  zeigeDichAn();
}

Teilchen::~Teilchen()
{
  delete umrissform;
}

void Teilchen::loescheDichWeg()
{
  //in dieser Klasse nicht noetig
}

void Teilchen::zeigeDichAn()
{
  if (image!=NULL)
  {
	zFlaeche->positioniereBild(image, ort);
  }
}

void Teilchen::setzeOrt(Vektor2D neuOrt)
{
  ort = neuOrt;
  zeigeDichAn();
}

void Teilchen::setzeGeschw(Vektor2D neuGeschw)
{
  geschw = neuGeschw;
  zeigeDichAn();
}

void Teilchen::setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw)
{
  ort = neuOrt;
  geschw = neuGeschw;
  zeigeDichAn();
}

void Teilchen::bewegeDich(float zeitdauer)
{
  geschw = geschw + beschl * zeitdauer;
  ort = ort + geschw * zeitdauer;

  zeigeDichAn();

  beschl = nullvektor;
}

void Teilchen::beeinflusstVonKraft(Vektor2D kraft)
{
  beschl = beschl + kraft / masse;
}

class Fixpunkt : public Objekt
{
  public:
	Fixpunkt(Zeichenflaeche2D *neuZFlaeche, TImage *neuImage,
			 float neuRueckprallFaktor, Vektor2D neuOrt);

	void bewegeDich(float zeitdauer);
	void beeinflusstVonKraft(Vektor2D kraft);

	void setzeOrt(Vektor2D neuOrt);
	void setzeGeschw(Vektor2D neuGeschw);
	void setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw);

	void loescheDichWeg();
	void zeigeDichAn();
};

Fixpunkt::Fixpunkt(Zeichenflaeche2D *neuZFlaeche, TImage* neuImage,
				   float neuRueckprallFaktor, Vektor2D neuOrt)
{
  zFlaeche = neuZFlaeche;
  image = neuImage;

  ort = neuOrt;
  masse = 1.0e30f;
  rueckprallFaktor = neuRueckprallFaktor;
  umrissform = NULL;

  zeigeDichAn();
}

void Fixpunkt::loescheDichWeg()
{
  //in dieser Klasse nicht noetig
}

void Fixpunkt::zeigeDichAn()
{
  zFlaeche->positioniereBild(image, ort);
}

void Fixpunkt::setzeOrt(Vektor2D neuOrt)
{
  //in dieser Klasse nicht ntig
}

void Fixpunkt::setzeGeschw(Vektor2D neuGeschw)
{
  //in dieser Klasse nicht ntig
}

void Fixpunkt::setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw)
{
  //in dieser Klasse nicht ntig
}

void Fixpunkt::bewegeDich(float zeitdauer)
{
  //in dieser Klasse nicht ntig
}

void Fixpunkt::beeinflusstVonKraft(Vektor2D kraft)
{
  //in dieser Klasse nicht ntig
}

class Fixwand : public Objekt
{
  public:
	Vektor2D ort2;
	bool sichtbar;
  public:
	Fixwand(Zeichenflaeche2D *neuZFlaeche, TImage *neuImage,
			float neuRueckprallFaktor,
			Vektor2D neuOrt1, Vektor2D neuOrt2, float neuDicke,
			bool neuSichtbar);

	void bewegeDich(float zeitdauer);
	void beeinflusstVonKraft(Vektor2D kraft);

	void setzeOrt(Vektor2D neuOrt);
	void setzeGeschw(Vektor2D neuGeschw);
	void setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw);

	void loescheDichWeg();
	void zeigeDichAn();
};

Fixwand::Fixwand(Zeichenflaeche2D *neuZFlaeche, TImage* neuImage,
				 float neuRueckprallFaktor,
				 Vektor2D neuOrt1, Vektor2D neuOrt2, float neuDicke,
				 bool neuSichtbar)
{
  zFlaeche = neuZFlaeche;
  image = neuImage;
  sichtbar = neuSichtbar;

  ort = neuOrt1;
  ort2 = neuOrt2;
  masse = 1.0e30f;
  rueckprallFaktor = neuRueckprallFaktor;
  umrissform = new Halbebene(nullvektor, ort2 - ort, neuDicke);

  zeigeDichAn();
}

void Fixwand::loescheDichWeg()
{
  //in dieser Klasse nicht noetig
}

void Fixwand::zeigeDichAn()
{
  if(sichtbar)
  {
	if(image == NULL)
	{
	  zFlaeche->setzeStiftFarbe(clBlue);
	  zFlaeche->setzeStiftDicke(2);
	  zFlaeche->zeichneLinie(ort, ort2);
	}
	else
	{
	  zFlaeche->positioniereBild(image, ort);
	}
  }
}

void Fixwand::setzeOrt(Vektor2D neuOrt)
{
  //in dieser Klasse nicht ntig
}

void Fixwand::setzeGeschw(Vektor2D neuGeschw)
{
  //in dieser Klasse nicht ntig
}

void Fixwand::setzeOrtGeschw(Vektor2D neuOrt, Vektor2D neuGeschw)
{
  //in dieser Klasse nicht ntig
}

void Fixwand::bewegeDich(float zeitdauer)
{
  //in dieser Klasse nicht ntig
}

void Fixwand::beeinflusstVonKraft(Vektor2D kraft)
{
  //in dieser Klasse nicht ntig
}

//-------------------------------------------------------------------------
//Physikklassen fr Krfte
//-------------------------------------------------------------------------
class Kraft
{
  public:
	Zeichenflaeche2D *zFlaeche;
  public:
	virtual void lasseKraftWirken(Objekt* objekt) = 0;
};

class Gravitationskraft : public Kraft
{
  public:
	Vektor2D beschl;
  public:
	Gravitationskraft(Vektor2D neuBeschl);
	void lasseKraftWirken(Objekt* objekt);
};

Gravitationskraft::Gravitationskraft(Vektor2D neuBeschl)
{
  beschl = neuBeschl;
}

void Gravitationskraft::lasseKraftWirken(Objekt* objekt)
{
  objekt->beeinflusstVonKraft(objekt->masse * beschl);
}

class Reibungskraft : public Kraft
{
  public:
	float alpha;
	float beta;
  public:
	Reibungskraft(float neuAlpha);
	Reibungskraft(float neuAlpha, float neuBeta);
	void lasseKraftWirken(Objekt* objekt);
};

Reibungskraft::Reibungskraft(float neuAlpha)
{
  alpha = neuAlpha;
  beta = 0;
}

Reibungskraft::Reibungskraft(float neuAlpha, float neuBeta)
{
  alpha = neuAlpha;
  beta = neuBeta;
}

void Reibungskraft::lasseKraftWirken(Objekt* objekt)
{
  Vektor2D geschw = objekt->geschw;
  float betragGeschw = betrag(geschw);
  objekt->beeinflusstVonKraft(-alpha * geschw - beta * betragGeschw * geschw);
}

class Federkraft : public Kraft
{
  public:
	Objekt* aufhaengungsobjekt;
	float federhaerte;
	float normallaenge;
  public:
	Federkraft(Objekt* neuAufhaengungsobjekt, float neuFederhaerte,
			   float neuNormallaenge);
	void lasseKraftWirken(Objekt* objekt);
};

Federkraft::Federkraft(Objekt* neuAufhaengungsobjekt, float neuFederhaerte,
					   float neuNormallaenge)
{
  aufhaengungsobjekt = neuAufhaengungsobjekt;
  federhaerte = neuFederhaerte;
  normallaenge = neuNormallaenge;
}

void Federkraft::lasseKraftWirken(Objekt* objekt)
{
  Vektor2D differenz = objekt->ort - aufhaengungsobjekt->ort;
  float laenge = betrag(differenz);
  Vektor2D eDifferenz = differenz / laenge;

  float dehnung = laenge - normallaenge;

  objekt->beeinflusstVonKraft(-federhaerte * dehnung * eDifferenz);
}

class Auftriebskraft : public Kraft
{
  public:
	float hoeheObjekt;   //in m
	float volumenObjekt; //in m
	float dichteWasser;  //hier in N/m, z.B.: 10000 N/m
	float yWasser;       //Position des Wasserspiegels
  public:
	Auftriebskraft(float neuHoeheObjekt, float neuVolumenObjekt,
				   float dichteWasser, float neuYWasser);
	void lasseKraftWirken(Objekt* objekt);
};

Auftriebskraft::Auftriebskraft(float neuHoeheObjekt, float neuVolumenObjekt,
							   float neuDichteWasser, float neuYWasser)
{
  hoeheObjekt = neuHoeheObjekt;
  volumenObjekt = neuVolumenObjekt;
  dichteWasser = neuDichteWasser;
  yWasser = neuYWasser;
}

void Auftriebskraft::lasseKraftWirken(Objekt* objekt)
{
  float yObjekt = objekt->ort.y;
  float yRelativ = yObjekt - yWasser;

  float halbeHoeheObjekt = 0.5f*hoeheObjekt;

  if(yRelativ < halbeHoeheObjekt)     //teilweise oder ganz im Wasser?
  {
	float yAuftriebskraft;
	if(yRelativ <= -halbeHoeheObjekt) //ganz unterhalb Wasser
	{
	  yAuftriebskraft = dichteWasser * volumenObjekt;
	}
	else                              //nur teilweise unter Wasser
	{
	  yAuftriebskraft = (halbeHoeheObjekt - yRelativ) / hoeheObjekt
							* dichteWasser * volumenObjekt;
	}
	objekt->beeinflusstVonKraft(Vektor2D(0.0f, yAuftriebskraft));
  }
}

//-------------------------------------------------------------------------
//Physikklassen fr Kontakte & Kollisionsdaten
//-------------------------------------------------------------------------
class Kontakt             //wird von Kollisionsdetektor generiert
{
  public:
	Vektor2D kontPkt;     //Kontaktstelle
	Vektor2D kontNormale; //Kontaktnormale (von Objekt1 nach Objekt2)
	float eindringtiefe;  //Eindringtiefe
  public:
	Kontakt();
	Kontakt(Vektor2D neuKontPkt, Vektor2D neuKontNormale,
			float neuEindringtiefe);
};

Kontakt::Kontakt()
{
}

Kontakt::Kontakt(Vektor2D neuKontPkt, Vektor2D neuKontNormale,
				 float neuEindringtiefe)
{
  kontPkt = neuKontPkt;
  kontNormale = neuKontNormale;
  eindringtiefe = neuEindringtiefe;
}

class Kollisionsdaten
{
  public:
	Objekt* objekt[2];  //Zeiger auf die zwei Objekte, die sich berhren
	int anzahlKontakte; //In dieser Version maximal 1 Kontakt pro Kollision  !!
	Kontakt kontakt[2]; //Folglich wird von diesem Kontakt-Feld
						//maximal 1 Element, d.h. kontakt[0] bentigt!
  public:
	Kollisionsdaten();
	Kollisionsdaten(Objekt* neuObjekt1, Objekt* neuObjekt2,
					Kontakt neuKontakt);
	Kollisionsdaten(Objekt* neuObjekt1, Objekt* neuObjekt2,
					Kontakt neuKontakt1, Kontakt neuKontakt2);
};

Kollisionsdaten::Kollisionsdaten()
{
  anzahlKontakte = 0;
}

Kollisionsdaten::Kollisionsdaten(Objekt* neuObjekt1, Objekt* neuObjekt2,
								 Kontakt neuKontakt)
{
  objekt[0] = neuObjekt1;
  objekt[1] = neuObjekt2;
  anzahlKontakte = 1;
  kontakt[0] = neuKontakt;
}

Kollisionsdaten::Kollisionsdaten(Objekt* neuObjekt1, Objekt* neuObjekt2,
								 Kontakt neuKontakt1, Kontakt neuKontakt2)
{
  objekt[0] = neuObjekt1;
  objekt[1] = neuObjekt2;
  anzahlKontakte = 2;
  kontakt[0] = neuKontakt1;
  kontakt[1] = neuKontakt2;
}

//-------------------------------------------------------------------------
//Physikklassen fr Kollisionsdetektor
//-------------------------------------------------------------------------
class Kollisionsmanager
{
  public:
	vector<Kollisionsdaten>kollisionsdaten;
  public:
	void ermittleKollisionen(vector<Objekt*> &objektliste);

	Kollisionsdaten objektUndObjekt(Objekt* objekt1, Objekt* objekt2);
	Kollisionsdaten halbebeneUndKreis(Objekt* objekt1, Objekt* objekt2);
	Kollisionsdaten kreisUndKreis(Objekt* objekt1, Objekt* objekt2);

	void beseitigeEindringungen();
	void beseitigeEindringung(Kollisionsdaten kollisionsdaten);

	void korrigiereBewegungen();
	void korrigiereBewegung(Kollisionsdaten kollisionsdaten);

	Objekt* sucheKollisionspartner(Objekt* objekt);
};

void Kollisionsmanager::ermittleKollisionen(vector<Objekt*> &objektliste)
{
  int i,j;
  Kollisionsdaten k;
  const int zahlObjekte = objektliste.size();

  kollisionsdaten.clear();

  for(i=0; i<zahlObjekte; i++)
  {
	for(j=i+1; j<zahlObjekte; j++)
	{
	  k = objektUndObjekt(objektliste[i], objektliste[j]);
	  if (k.anzahlKontakte > 0)
	  {
		kollisionsdaten.push_back(k);
	  }
	}
  }
}

Kollisionsdaten Kollisionsmanager::objektUndObjekt(Objekt* objekt1,
													Objekt* objekt2)
{
  if (objekt1->umrissform == NULL || objekt2->umrissform == NULL)
  {
	return Kollisionsdaten();
  }

  int umrissform1 = objekt1->umrissform->typ;
  int umrissform2 = objekt2->umrissform->typ;

  if (umrissform1 > umrissform2)
  {
	Objekt* hObjekt = objekt2;
	objekt2 = objekt1;
	objekt1 = hObjekt;

	int hUmrissform = umrissform2;
	umrissform2 = umrissform1;
	umrissform1 = hUmrissform;
  }

  if (umrissform1 == halbebene)
  {
	if (umrissform2 == kreis)
	{
	  return halbebeneUndKreis(objekt1, objekt2);
	}
	//Kollisionen von Halbebene mit Halbebene nicht implementiert!
  }
  else if(umrissform1 == kreis)
  {
	if (umrissform2 == kreis)
	{
	  return kreisUndKreis(objekt1, objekt2);
	}
  }
  return Kollisionsdaten();
}

Kollisionsdaten Kollisionsmanager::halbebeneUndKreis(Objekt* objekt1,
												  Objekt* objekt2)
{
  Halbebene* halbebene = (Halbebene*)(objekt1->umrissform);
  Kreis* kreis = (Kreis*)(objekt2->umrissform);

  Vektor2D halbebenePkt1 = objekt1->ort + halbebene->pkt1;
  Vektor2D halbebenePkt2 = objekt1->ort + halbebene->pkt2;
  Vektor2D halbebeneDifferenz = halbebene->pkt1ZuPkt2;
  Vektor2D halbebeneDifferenzEinheit = halbebene->pkt1ZuPkt2Einheit;
  Vektor2D halbebeneNormale = halbebene->normale;

  Vektor2D kreisMittelpunkt = objekt2->ort;
  float kreisRadius = kreis->radius;

  Vektor2D pkt1ZuMittelpkt = kreisMittelpunkt - halbebenePkt1;
  float projAbstand = pkt1ZuMittelpkt * halbebeneNormale;

  float skalarprodukt = pkt1ZuMittelpkt * halbebeneDifferenz;
  float betragQuadrat = halbebene->laenge * halbebene->laenge;

  if (projAbstand > kreisRadius || projAbstand < -halbebene->dicke)
  {
	return Kollisionsdaten();
  }
  else if (skalarprodukt < 0.0f || skalarprodukt > betragQuadrat)
  {
	return Kollisionsdaten();
  }
  else
  {
	Vektor2D mittelpktZuKontPkt = projAbstand * halbebeneNormale;
	Vektor2D kontPkt = halbebenePkt1 +  pkt1ZuMittelpkt + mittelpktZuKontPkt;
	float eindringtiefe = kreisRadius - projAbstand;

	Kontakt kontakt(kontPkt, halbebeneNormale, eindringtiefe);
	return Kollisionsdaten(objekt1, objekt2, kontakt);
  }
}

Kollisionsdaten Kollisionsmanager::kreisUndKreis(Objekt* objekt1,
												  Objekt* objekt2)
{
  Kreis* kreis1 = (Kreis*)(objekt1->umrissform);
  Kreis* kreis2 = (Kreis*)(objekt2->umrissform);

  float radius1 = kreis1->radius;
  float radius2 = kreis2->radius;

  Vektor2D differenz = objekt2->ort - objekt1->ort;
  float abstand = betrag(differenz);

  if (abstand > radius1 + radius2)
  {
	return Kollisionsdaten();
  }
  else
  {
	Vektor2D kontPkt = 0.5f * (objekt1->ort + objekt2->ort);
	Vektor2D kontNormale = differenz / abstand;
	float eindringtiefe = radius1 + radius2 - abstand;

	Kontakt kontakt(kontPkt, kontNormale, eindringtiefe);
	return Kollisionsdaten(objekt1, objekt2, kontakt);
  }
}

void Kollisionsmanager::beseitigeEindringung(Kollisionsdaten kollisionsdaten)
{
  Objekt* objekt1 = kollisionsdaten.objekt[0];
  Objekt* objekt2 = kollisionsdaten.objekt[1];

  if (kollisionsdaten.anzahlKontakte == 1)
  {
	float masse1 = objekt1->masse;
	float masse2 = objekt2->masse;

	float anteil1 = masse2 / (masse1 + masse2);
	float anteil2 = 1.0f - anteil1;

	float eindringtiefe = kollisionsdaten.kontakt[0].eindringtiefe;
	Vektor2D kontNormale = kollisionsdaten.kontakt[0].kontNormale;

	Vektor2D neuOrt1 = objekt1->ort - eindringtiefe * anteil1 * kontNormale;
	Vektor2D neuOrt2 = objekt2->ort + eindringtiefe * anteil2 * kontNormale;

	objekt1->setzeOrt(neuOrt1);
	objekt2->setzeOrt(neuOrt2);
  }
  else if (kollisionsdaten.anzahlKontakte == 0)
  {
	return;
  }
  else
  {
	return; //mehr als 1 Kontakt: nicht implementiert!
  }
}

void Kollisionsmanager::beseitigeEindringungen()
{
  unsigned int i;
  for(i=0; i<kollisionsdaten.size(); i++)
  {
	beseitigeEindringung(kollisionsdaten[i]);
  }
}

void Kollisionsmanager::korrigiereBewegung(Kollisionsdaten kollisionsdaten)
{
  Objekt* objekt1 = kollisionsdaten.objekt[0];
  Objekt* objekt2 = kollisionsdaten.objekt[1];

  if (kollisionsdaten.anzahlKontakte == 1)
  {
	float masse1 = objekt1->masse;
	float masse2 = objekt2->masse;

	float anteil1 = masse1 / (masse1 + masse2);
	float anteil2 = 1.0f - anteil1;

	Vektor2D geschw1 = objekt1->geschw;
	Vektor2D geschw2 = objekt2->geschw;
	Vektor2D kontNormale = kollisionsdaten.kontakt[0].kontNormale;

	float projGeschw1Normale = geschw1 * kontNormale;
	float projGeschw2Normale = geschw2 * kontNormale;
	Vektor2D geschw1Normale = projGeschw1Normale * kontNormale;
	Vektor2D geschw1Parallel = geschw1 - geschw1Normale;
	Vektor2D geschw2Normale = projGeschw2Normale * kontNormale;
	Vektor2D geschw2Parallel = geschw2 - geschw2Normale;

	float projGeschw1NormaleNachher =
	  anteil1 * projGeschw1Normale +
	  anteil2 * (2 * projGeschw2Normale - projGeschw1Normale);
	float projGeschw2NormaleNachher =
	  anteil2 * projGeschw2Normale +
	  anteil1 * (2 * projGeschw1Normale - projGeschw2Normale);

	float rueckprallFaktor = 0.5f * (objekt1->rueckprallFaktor +
									 objekt2->rueckprallFaktor);

	Vektor2D geschw1Nachher = rueckprallFaktor *
								 projGeschw1NormaleNachher * kontNormale +
								 geschw1Parallel;
	Vektor2D geschw2Nachher = rueckprallFaktor *
								 projGeschw2NormaleNachher * kontNormale +
								 geschw2Parallel;

	objekt1->setzeGeschw(geschw1Nachher);
	objekt2->setzeGeschw(geschw2Nachher);
 }
}

void Kollisionsmanager::korrigiereBewegungen()
{
  unsigned int i;
  for(i=0; i<kollisionsdaten.size(); i++)
  {
	korrigiereBewegung(kollisionsdaten[i]);
  }
}

Objekt* Kollisionsmanager::sucheKollisionspartner(Objekt* objekt)
{
  int i;

  const int zahlKollisionen = kollisionsdaten.size();

  for(i=0; i<zahlKollisionen; i++)
  {
	if(kollisionsdaten[i].objekt[0] == objekt)
	{
	  return kollisionsdaten[i].objekt[1];
	}
	if(kollisionsdaten[i].objekt[1] == objekt)
	{
	  return kollisionsdaten[i].objekt[0];
	}
  }
  return NULL;
}

//-------------------------------------------------------------------------
//Physikklassen fr Objekt-Kraft-Liste
//-------------------------------------------------------------------------
class ObjektKraftListeneintrag
{
  public:
	Objekt* objekt;
	Kraft* kraft;
  public:
	ObjektKraftListeneintrag(Objekt* neuObjekt, Kraft* neuKraft);
};

ObjektKraftListeneintrag::ObjektKraftListeneintrag(Objekt* neuObjekt,
												   Kraft* neuKraft)
{
  objekt = neuObjekt;
  kraft = neuKraft;
}

class ObjektKraftListe
{
  public:
    vector<ObjektKraftListeneintrag*> liste;
  public:
    ObjektKraftListe();
    ~ObjektKraftListe();
    void fuegeHinzu(Objekt* neuObjekt, Kraft* neuKraft);
    void entferne(Objekt* objekt, Kraft* kraft);
    void entferneAlle();
    void lasseAlleKraefteWirken();
};

ObjektKraftListe::ObjektKraftListe()
{
}

ObjektKraftListe::~ObjektKraftListe()
{
  unsigned int i;
  for(i=0; i<liste.size(); i++)
  {
    delete liste[i];
  }
}

void ObjektKraftListe::fuegeHinzu(Objekt* neuObjekt, Kraft* neuKraft)
{
  liste.push_back(new ObjektKraftListeneintrag(neuObjekt, neuKraft));
}

void ObjektKraftListe::entferne(Objekt* objekt, Kraft* kraft)
{
  unsigned int i;
  for(i=0; i<liste.size(); i++)
  {
    if (liste[i]->objekt == objekt && liste[i]->kraft == kraft)
    {
      delete liste[i];
      liste.erase(&liste[i]);
      return;
    }
  }
}

void ObjektKraftListe::entferneAlle()
{
  unsigned int i;
  for(i=0; i<liste.size(); i++)
  {
    delete liste[i];
  }
  liste.clear();
}

void ObjektKraftListe::lasseAlleKraefteWirken()
{
  unsigned int i;
  for(i=0; i<liste.size(); i++)
  {
	liste[i]->kraft->lasseKraftWirken(liste[i]->objekt);
  }
}

//-------------------------------------------------------------------------
//Physikklassen fr die Szene
//-------------------------------------------------------------------------
class Szene
{
  public:
	Zeichenflaeche2D* zFlaeche;

	vector<Objekt*> objektliste;
	ObjektKraftListe* objektKraftListe;
	Kollisionsmanager *kollisionsmanager;

  public:
	Szene(Zeichenflaeche2D* neuZFlaeche);
	~Szene();

	void fuegeHinzuObjekt(Objekt* neuObjekt);
	void entferneObjekt(Objekt* objekt);

	void bewegeDich(float zeitdauer);
};

Szene::Szene(Zeichenflaeche2D* neuZFlaeche)
{
  zFlaeche = neuZFlaeche;

  objektKraftListe = new ObjektKraftListe();
  kollisionsmanager = new Kollisionsmanager();
}

Szene::~Szene()
{
  delete kollisionsmanager;
  delete objektKraftListe;
}

void Szene::fuegeHinzuObjekt(Objekt* neuObjekt)
{
  objektliste.push_back(neuObjekt);
}

void Szene::entferneObjekt(Objekt* objekt)
{
  unsigned int i;

  for(i=0; i<objektliste.size(); i++)
  {
	if(objektliste[i] == objekt)
	{
	  objektliste.erase(&objekt);
	  return;
	}
  }
}

void Szene::bewegeDich(float zeitdauer)
{
  unsigned int i;


  objektKraftListe->lasseAlleKraefteWirken();
  for(i=0; i<objektliste.size(); i++)
  {
	objektliste[i]->bewegeDich(zeitdauer);
  }

  kollisionsmanager->ermittleKollisionen(objektliste);
  kollisionsmanager->beseitigeEindringungen();
  kollisionsmanager->korrigiereBewegungen();
}
